<?php
/*
 * Copyright (c) 2013, webvariants GbR, http://www.webvariants.de
 *
 * This file is released under the terms of the MIT license. You can find the
 * complete text in the attached LICENSE file or online at:
 *
 * http://www.opensource.org/licenses/mit-license.php
 */

/**
 * Datentyp "Datum & Zeit"
 */
class WV_Datatype_DateTime extends WV_Datatype_Base implements WV_IDatatype {
	public function renderForm(sly_Form $form, WV_IProperty $property, $value) {
		$value    = $this->decodeFormValue($property, $value);
		$params   = $this->deserializeParams($property->getParams());
		$style    = sly_Util_String::startsWith($params['style'], 'date') ? 'date' : $params['style'];
		$required = !empty($params['required']);
		$disabled = !empty($params['disabled']);
		$readonly = !empty($params['readonly']);

		if (array_key_exists('sortable', $params) && $params['sortable']) {
			$value = $this->decodeSortable($value);
		}

		if ($style === 'date') {
			$withTime = !empty($params['withtime']) || $params['style'] === 'datetime' || $params['style'] === 'datetime-local';
			$input    = new sly_Form_DateTime('property_'.$property->getID().'_value', $property->getTitle(), $value, null, $withTime);
		}
		elseif (in_array($style, array('month', 'week', 'time'))) {
			if ($value !== null && $value !== '') {
				$value = intval($value);
				switch ($style) {
					case 'month':
						$value = date('Y-m', $value);
						break;
					case 'week':
						$value = date('Y-\WW', $value);
						break;

					case 'time':
						$value = date('H:i:s', $value);
						break;
				}
			}
			$input = new sly_Form_Input_Text('property_'.$property->getID().'_value', $property->getTitle(), $value, null);
			$input->setAttribute('type', $style);
		}
		else {
			throw new WV_Exception_Datatype('Cannot render datetime datatype with unknown style: '.sly_html($style));
		}

		$input->setMultilingual($property->isMultilingual());
		if ($required) $input->setRequired(true);
		if ($disabled) $input->setDisabled(true);
		if ($readonly) $input->setAttribute('readonly', 'readonly');

		if (!empty($params['hide'])) $input->addFormRowClass('hide');

		WV_Datatype::setHelpTextOnElement($form, $input, $property);
		$form->add($input);

		return $form;
	}

	protected function getDefaultParams() {
		return array(
			'withtime' => false, /* deprecated */
			'style'    => 'date'
		);
	}

	public function serializeValue($value, array $params) {
		if (array_key_exists('strtotime', $params) && $params['strtotime']) {
			if ($value === null || !is_scalar($value) || strlen($value) === 0) {
				$value = '';
			}
			elseif (!ctype_digit((string) $value)) {
				$time = strtotime($value);
				if ($time === false) {
					$value = '';
				} else {
					$value = $time;
				}
			}
		}

		if (array_key_exists('sortable', $params) && $params['sortable']) {
			$value = $this->encodeSortable($value);
		}

		return $value;
	}

	public function deserializeValue($value, array $params) {
		if (array_key_exists('sortable', $params) && $params['sortable']) {
			$value = $this->decodeSortable($value);
		}

		// only the empty string means 'not given', 0 can actually mean a valid date!
		return $value === '' || $value === null ? null : (int) $value;
	}

	public function serializeFormFilter($list, WV_IProperty $property) {
		$params = $property->getParams();
		foreach ($list as $idx => $value) {
			$tstamp = $value === '' ? '' : strtotime($value);

			if ($tstamp === false) {
				throw new WV_Exception_Datatype('Cannot convert from string to time ('.$value.').');
			}

			if (array_key_exists('sortable', $params) && $params['sortable']) {
				$tstamp = $this->encodeSortable($tstamp);
			}

			$list[$idx] = $tstamp;
		}

		return $list;
	}

	private static function encodeSortable($value) {
		if (is_scalar($value) && strlen($value)) {

			if (substr($value, 0, 1) === '-') {
				$sub = substr($value, 1);
				if (ctype_digit($sub)) {
					$value = '-' . str_pad($sub, 12, '0', STR_PAD_LEFT);
				}
			} else {
				if (ctype_digit($value)) {
					$value = str_pad($value, 13, '0', STR_PAD_LEFT);
				}
			}
		}

		return $value;
	}

	private static function decodeSortable($value) {
		if (is_scalar($value) && strlen($value)) {
			if (substr($value, 0, 1) === '-') {
				$sub = substr($value, 1);
				if (ctype_digit($sub)) {
					$value = '-' . ltrim($sub, '0');
					if ($value === '-') {
						$value = '-0';
					}
				}
			} else {
				if (ctype_digit($value)) {
					$value = ltrim($value, '0');
					if ($value === '') {
						$value = '0';
					}
				}
			}
		}

		return $value;
	}
}
