<?php
/*
 * Copyright (c) 2013, webvariants GbR, http://www.webvariants.de
 *
 * This file is released under the terms of the MIT license. You can find the
 * complete text in the attached LICENSE file or online at:
 *
 * http://www.opensource.org/licenses/mit-license.php
 */

namespace wv\metainfo\Property;

use wv\metainfo\Metainfo;

/**
 * Metainformation
 *
 * Eine Metainformation ist eine zusätzliche Information, die einem Artikel /
 * Kategorie / Medium / Benutzer zugewiesen werden kann. Sie besteht aus einem
 * internen Namen, einem Ttel, einem Datentyp sowie einigen weiteren Daten.
 *
 * Metainformationen können über Artikeltypen gruppiert werden. Auf diese Weise
 * kann einem Artikel immer eine ganz bestimmte Teilmenge aller verfügbaren
 * Metainformationen zugewiesen werden.
 *
 * Eine einzelne Metainformation kann nur entweder für Artikel oder für Kategorien
 * oder für Medien oder für Benutzer verwendet werden. Die internen Namen müssen
 * für jeden dieser drei Bereiche jeweils eindeutig sein.
 *
 * Artikel-Metainformationen können noch einmal in zwei Bereiche unterteilt
 * werden:
 *
 *	- Content-Seite: Die Information (das Formular) wird direkt auf der
 *	  Slice-Seite eines Artikels angezeigt.
 *	- Extra-Seite: Die Information wird auf der separaten Meta-Seite von
 *	  Sally angezeigt.
 *
 * @ingroup objects
 * @author	Christoph
 */
class Property extends \WV_Property {
	protected $type;    ///< int  Typ (Artikel-, Kategorie- oder Medium-Metainformation)
	protected $pos;
	protected $group;
	protected $backend; // show metadata in backend forms

	/**
	 * Constructor
	 *
	 * @param type  $type
	 * @param type  $name
	 * @param array $data
	 */
	public function __construct($type, $name, array $data) {
		parent::__construct($name, $data);

		$this->type         = $type; // wv\metainfo\Metainfo::TYPE_ARTICLE, ...
		$this->pos          = $this->getData('pos', null);
		$this->group        = $this->getData('group', '');
		$this->backend      = $this->getData('backend', true);
		$this->multilingual = isset($data['multilingual']) ? $data['multilingual'] : false;
	}

	public static function factory($type, $name, array $property) {
		switch ($type) {
			case Metainfo::TYPE_ARTICLE:
				return new Article($name, $property);

			case Metainfo::TYPE_CATEGORY:
				return new Category($name, $property);

			case Metainfo::TYPE_USER:
				return new User($name, $property);

			case Metainfo::TYPE_MEDIUM:
				return new Property($type, $name, $property);

			default:
				throw new Exception('Unknown metainfo type '.$type.' given.');
		}
	}

	public function getId() {
		return sprintf('%u', crc32($this->type.$this->name));
	}

	public function getType() {
		return $this->type;
	}

	public function getPosition() {
		return $this->pos;
	}

	public function getGroup() {
		return $this->group;
	}

	public function getBackend() {
		return $this->backend;
	}
}
