<?php
/*
 * Copyright (c) 2013, webvariants GbR, http://www.webvariants.de
 *
 * This file is released under the terms of the MIT license. You can find the
 * complete text in the attached LICENSE file or online at:
 *
 * http://www.opensource.org/licenses/mit-license.php
 */

namespace wv\realurl2\Listener;

use sly_Core;
use sly_Form;
use sly_Model_Article;
use sly_Model_Base_Article;
use sly_Util_Language;
use wv\realurl2\RealURL2;
use wv\realurl2\Util;
use wv\realurl2\Cache\Persistent as Cache;

/**
 * Sally-Eventlistener
 *
 * Diese Klasse sammelt zu einem Großteil nur Listener, um den URL-Cache immer
 * aktuell zu halten. Für gewöhnlich will niemand diese Methoden direkt
 * aufrufen.
 *
 * @author Christoph
 * @since  forever
 */
class Articles extends Base {
	/**
	 * SLY_ART_ADDED: article was created
	 * SLY_ART_COPIED: article was copied
	 */
	public function artAdded(sly_Model_Base_Article $article) {
		if (!Util::hasSiteStartArticle()) return;
		$this->rebuildArticle($article);
	}

	/**
	 * SLY_ART_UPDATED: article was renamed
	 */
	public function artUpdated(sly_Model_Article $article) {
		if (!Util::hasSiteStartArticle()) return;

		// do nothing if on contentmeta page, since we do the work in artMetaUpdated()
		if (sly_Core::getCurrentControllerName() === 'contentmeta') return;

		// the name of a category's start article is irrelevant, so do nothing if this is a start article
		if ($article->isStartArticle()) return;

		// rebuild
		$this->rebuildArticle($article);
	}

	/**
	 * SLY_ART_DELETED: article was deleted
	 */
	public function artDeleted(sly_Model_Article $article) {
		if (!Util::hasSiteStartArticle()) return;

		$id      = $article->getId();
		$storage = $this->getStorage();

		$storage->clearByArticle($id); // affects *all* URLs to this article
	}

	/**
	 * SLY_ART_TOUCHED
	 */
	public function artTouched(sly_Model_Article $article, array $params) {
		$source      = $params['source'];
		$fragment    = Util::getFragment($source);
		$skip        = Util::isSkipped($source) ? 1 : 0;
		$articleID   = $article->getId();
		$languageID  = $article->getClang();
		$newRevision = $article->getRevision();
		$database    = $this->container['sly-persistence'];

		// copy fragment and skip status

		$database->update(
			'article',
			array('wv24_fragment' => $fragment, 'wv24_skip' => $skip),
			array('id' => $articleID, 'clang' => $languageID, 'revision' => $newRevision)
		);

		// update article revision
		// (In case no fancy onlinecontrol addOn is installed, the newly touched
		// revision is online immediately)

		if ($article->isOnline()) {
			$database->update(
				'local_realurl2',
				array('revision' => $newRevision),
				array('article_id' => $articleID, 'clang' => $languageID)
			);
		}

		// clear cache for this article
		Cache::clear(RealURL2::URL_NAMESPACE, $articleID.'_'.$languageID);
	}

	/**
	 * SLY_ART_ONLINE: another article revision has been set to online
	 */
	public function artOnline(sly_Model_Article $article) {
		if (!Util::hasSiteStartArticle()) return;
		$this->rebuildArticle($article);
	}

	/**
	 * SLY_ART_OFFLINE: an article went offline
	 */
	public function artOffline(sly_Model_Article $article) {
		if (!Util::hasSiteStartArticle()) return;
		$this->rebuildArticle($article);
	}

	/**
	 * SLY_ART_META_FORM_FIELDSET: Fragment-Formular erzeugen
	 *
	 * @param  sly_Form $form    the form object to work on
	 * @param  array    $params  die von Sally übergebenen Parameter
	 * @return sly_Form          das übergebene Subject mit dem angehängten Formularteil
	 */
	public function artMetaFormSection(sly_Form $form, array $params) {
		$article  = $params['article'];
		$fragment = Util::getFragment($article);

		require _WV24_PATH.'views/meta.phtml';

		return $form;
	}

	/**
	 * SLY_ART_META_UPDATED: Metadaten eines Artikels wurden aktualisiert
	 *
	 * @param sly_Model_Article $article  the article which meta data has been updated
	 */
	public function artMetaUpdated(sly_Model_Article $article) {
		$articleID = $article->getId();
		$clang     = $article->getClang();
		$request   = $this->container['sly-request'];

		if ($articleID === sly_Core::getSiteStartArticleId()) {
			return;
		}

		// do nothing if no realurl2 form was submitted
		if (!$request->post->has('realurl2_form')) {
			return;
		}

		$fragment = $this->getFragment();
		$db       = $this->container['sly-persistence'];

		// update database; set the fragment to all revisions, so it's easily
		// available when generating URLs (otherwise we would have to look at
		// rev 0 or something like that).
		$db->update('article', array('wv24_fragment' => $fragment), array('id' => $articleID, 'clang' => $clang));

		if (Util::hasSiteStartArticle()) {
			if ($article->isStartArticle()) {
				$this->rebuildTree($articleID, $clang, $article->getPath());
			}
			else {
				$this->rebuildArticle($article);
			}
		}

		$this->container['sly-flash-message']->appendInfo('realURL2-Einstellungen wurden aktualisiert.');
	}

	/**
	 * SLY_ART_MOVED
	 */
	public function artMoved($articleID, array $params) {
		if (!Util::hasSiteStartArticle()) return;

		$clang = $params['clang'];

		// kill all outdated URLs
		$storage = $this->getStorage();
		$storage->clearByArticle($articleID, $clang);

		// rebuild
		$this->rebuildSingle($articleID, $clang);
	}

	/**
	 * SLY_ART_TO_STARTPAGE
	 */
	public function artToStartpage($starterID, array $params) {
		if (!Util::hasSiteStartArticle()) return;

		$oldStarter = $params['old_cat'];

		// kill all outdated URLs
		$storage = $this->getStorage();
		$storage->clearByPath(RealURL2::URL_NAMESPACE, '%|'.$oldStarter.'|%');
		$storage->clearByArticle($oldStarter);

		// re-generate the tree
		foreach (sly_Util_Language::findAll(true) as $clang) {
			$this->rebuildTree($starterID, $clang, null, false);
		}
	}
}
